@@ -6,8 +6,7 @@
 #
 # DeVeDe-NG is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# the Free Software Foundation; either version 3 of the License.
 #
 # DeVeDe-NG is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -50,6 +49,7 @@ class avconv(devedeng.avbase.avbase):
                 mpeg2 = False
                 divx = False
                 h264 = False
+                hevc = False
                 for line in stdout.decode("latin-1").split("\n"):
                     parts = line.strip().split(" ")
                     if len(parts) < 2:
@@ -80,6 +80,9 @@ class avconv(devedeng.avbase.avbase):
                     if (codec == "h264") or (codec == "H264"):
                         h264 = True
                         continue
+                    if (codec == "hevc") or (codec == "HEVC"):
+                        hevc = True
+                        continue
                     if (codec == "mpeg4"):
                         divx = True
                         continue
@@ -95,6 +98,8 @@ class avconv(devedeng.avbase.avbase):
                     devedeng.avconv.avconv.disc_types.append("divx")
                 if (h264 and mp3):
                     devedeng.avconv.avconv.disc_types.append("mkv")
+                if (hevc and mp3):
+                    devedeng.ffmpeg.ffmpeg.disc_types.append("hevc")
 
                 return True
             else:
@@ -173,6 +178,20 @@ class avconv(devedeng.avbase.avbase):
             vflip = False
             hflip = False
 
+            if (file_project.prerotation == 90):
+                if (cmd_line != ""):
+                    cmd_line += ",fifo,"
+                cmd_line += "transpose=1"
+            elif (file_project.prerotation == -90) or (file_project.prerotation == 270):
+                if (cmd_line != ""):
+                    cmd_line += ",fifo,"
+                cmd_line += "transpose=2"
+            elif (file_project.prerotation == 180):
+                if (cmd_line != ""):
+                    cmd_line += ",fifo,"
+                vflip = not vflip
+                hflip = not hflip
+
             if (file_project.rotation == "rotation_90"):
                 if (cmd_line != ""):
                     cmd_line += ",fifo,"
@@ -182,8 +201,8 @@ class avconv(devedeng.avbase.avbase):
                     cmd_line += ",fifo,"
                 cmd_line += "transpose=2"
             elif (file_project.rotation == "rotation_180"):
-                vflip = True
-                hflip = True
+                vflip = not vflip
+                hflip = not hflip
 
             if (file_project.mirror_vertical):
                 vflip = not vflip
@@ -250,9 +269,9 @@ class avconv(devedeng.avbase.avbase):
                 if minrate < (devedeng.project.devede_project.divx_min_bps * 1000):
                     minrate = devedeng.project.devede_project.divx_min_bps * 1000
                 self.command_var.append(str(minrate))
-        elif (self.config.disc_type == "mkv"):
+        elif (self.config.disc_type == "mkv") or (self.config.disc_type == "hevc"):
             self.command_var.append("-vcodec")
-            self.command_var.append("h264")
+            self.command_var.append("h264" if self.config.disc_type == "mkv" else "hevc")
             self.command_var.append("-acodec")
             self.command_var.append("libmp3lame")
             self.command_var.append("-f")
@@ -424,11 +443,11 @@ class avconv(devedeng.avbase.avbase):
             else:
                 keyintv = 15
 
-            if (not file_project.gop12) and (not(self.config.disc_type == "divx")) and (not(self.config.disc_type == "mkv")):
+            if (not file_project.gop12) and (not(self.config.disc_type == "divx")) and (not(self.config.disc_type == "mkv") and (not(self.config.disc_type == "hevc"))):
                 self.command_var.append("-g")
                 self.command_var.append(str(keyintv))
 
-            if (self.config.disc_type == "divx") or (self.config.disc_type == "mkv"):
+            if (self.config.disc_type == "divx") or (self.config.disc_type == "mkv") or (self.config.disc_type == "hevc"):
                 self.command_var.append("-g")
                 self.command_var.append("300")
             elif file_project.gop12 and (not file_project.no_reencode_audio_video):
@@ -445,7 +464,7 @@ class avconv(devedeng.avbase.avbase):
             self.command_var.append(str(video_length))
 
         self.command_var.append("-ac")
-        if (file_project.sound5_1) and ((self.config.disc_type == "dvd") or (self.config.disc_type == "divx") or (self.config.disc_type == "mkv")):
+        if (file_project.sound5_1) and ((self.config.disc_type == "dvd") or (self.config.disc_type == "divx") or (self.config.disc_type == "mkv") or (self.config.disc_type == "hevc")):
             self.command_var.append("6")
         else:
             self.command_var.append("2")
@@ -478,7 +497,7 @@ class avconv(devedeng.avbase.avbase):
 
         if (vcd == False) and (file_project.no_reencode_audio_video == False):
             self.command_var.append("-b:a")
-            self.command_var.append(str(file_project.audio_rate_final*1000))
+            self.command_var.append(str(self._adjust_audio_bitrate(file_project.audio_rate_final)*1000))
 
             self.command_var.append("-b:v")
             self.command_var.append(str(file_project.video_rate_final*1000))
@@ -494,6 +513,13 @@ class avconv(devedeng.avbase.avbase):
 
         self.command_var.append(output_file)
 
+    def _adjust_audio_bitrate(self, bitrate):
+        br = [32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384]
+        for a in br:
+            if a >= bitrate:
+                return a
+        return 384
+
     def create_menu_mpeg(self, n_page, background_music, sound_length, pal, video_rate, audio_rate, output_path, use_mp2, widescreen):
 
         self.n_page = n_page
@@ -537,7 +563,7 @@ class avconv(devedeng.avbase.avbase):
         self.command_var.append("-b:v")
         self.command_var.append(str(video_rate) + "k")
         self.command_var.append("-b:a")
-        self.command_var.append(str(audio_rate) + "k")
+        self.command_var.append(str(self._adjust_audio_bitrate(audio_rate)) + "k")
         self.command_var.append("-aspect")
         if widescreen:
             self.command_var.append("16:9")
@@ -572,8 +598,11 @@ class avconv(devedeng.avbase.avbase):
                 parts = data[0][pos:pos2].split(":")
                 t = 0.0
                 for e in parts:
-                    t *= 60.0
-                    t += float(e)
+                    try:
+                        v = float(e)
+                    except:
+                        continue
+                    t = t * 60.0 + v
                 t /= self.final_length
                 self.progress_bar[1].set_fraction(t)
                 self.progress_bar[1].set_text("%.1f%%" % (100.0 * t))
